@@ -10,8 +10,8 @@
  * which can be found at <https://www.eyrie.org/~eagle/software/rra-c-util/>.
  *
  * Written by Russ Allbery <eagle@eyrie.org>
- * Copyright 2017 Russ Allbery <eagle@eyrie.org>
- * Copyright 2011, 2012, 2014
+ * Copyright 2017-2018, 2020 Russ Allbery <eagle@eyrie.org>
+ * Copyright 2011-2012, 2014
  *     The Board of Trustees of the Leland Stanford Junior University
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,6 +31,8 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
+ *
+ * SPDX-License-Identifier: MIT
  */
 
 #include <config.h>
@@ -57,12 +59,14 @@ static const struct {
     pam_call call;
     enum group_type group;
 } CALLS[] = {
-    { "acct_mgmt",     pam_sm_acct_mgmt,     GROUP_ACCOUNT  },
-    { "authenticate",  pam_sm_authenticate,  GROUP_AUTH     },
-    { "setcred",       pam_sm_setcred,       GROUP_AUTH     },
-    { "chauthtok",     pam_sm_chauthtok,     GROUP_PASSWORD },
-    { "open_session",  pam_sm_open_session,  GROUP_SESSION  },
-    { "close_session", pam_sm_close_session, GROUP_SESSION  },
+    /* clang-format off */
+    {"acct_mgmt",     pam_sm_acct_mgmt,     GROUP_ACCOUNT },
+    {"authenticate",  pam_sm_authenticate,  GROUP_AUTH    },
+    {"setcred",       pam_sm_setcred,       GROUP_AUTH    },
+    {"chauthtok",     pam_sm_chauthtok,     GROUP_PASSWORD},
+    {"open_session",  pam_sm_open_session,  GROUP_SESSION },
+    {"close_session", pam_sm_close_session, GROUP_SESSION },
+    /* clang-format on */
 };
 
 /* Mapping of PAM flag names without the leading PAM_ to values. */
@@ -70,15 +74,17 @@ static const struct {
     const char *name;
     int value;
 } FLAGS[] = {
-    { "CHANGE_EXPIRED_AUTHTOK", PAM_CHANGE_EXPIRED_AUTHTOK },
-    { "DISALLOW_NULL_AUTHTOK",  PAM_DISALLOW_NULL_AUTHTOK  },
-    { "DELETE_CRED",            PAM_DELETE_CRED            },
-    { "ESTABLISH_CRED",         PAM_ESTABLISH_CRED         },
-    { "PRELIM_CHECK",           PAM_PRELIM_CHECK           },
-    { "REFRESH_CRED",           PAM_REFRESH_CRED           },
-    { "REINITIALIZE_CRED",      PAM_REINITIALIZE_CRED      },
-    { "SILENT",                 PAM_SILENT                 },
-    { "UPDATE_AUTHTOK",         PAM_UPDATE_AUTHTOK         },
+    /* clang-format off */
+    {"CHANGE_EXPIRED_AUTHTOK", PAM_CHANGE_EXPIRED_AUTHTOK},
+    {"DISALLOW_NULL_AUTHTOK",  PAM_DISALLOW_NULL_AUTHTOK },
+    {"DELETE_CRED",            PAM_DELETE_CRED           },
+    {"ESTABLISH_CRED",         PAM_ESTABLISH_CRED        },
+    {"PRELIM_CHECK",           PAM_PRELIM_CHECK          },
+    {"REFRESH_CRED",           PAM_REFRESH_CRED          },
+    {"REINITIALIZE_CRED",      PAM_REINITIALIZE_CRED     },
+    {"SILENT",                 PAM_SILENT                },
+    {"UPDATE_AUTHTOK",         PAM_UPDATE_AUTHTOK        },
+    /* clang-format on */
 };
 
 /* Mapping of strings to PAM groups. */
@@ -86,10 +92,12 @@ static const struct {
     const char *name;
     enum group_type group;
 } GROUPS[] = {
-    { "account",  GROUP_ACCOUNT  },
-    { "auth",     GROUP_AUTH     },
-    { "password", GROUP_PASSWORD },
-    { "session",  GROUP_SESSION  },
+    /* clang-format off */
+    {"account",  GROUP_ACCOUNT },
+    {"auth",     GROUP_AUTH    },
+    {"password", GROUP_PASSWORD},
+    {"session",  GROUP_SESSION },
+    /* clang-format on */
 };
 
 /* Mapping of strings to PAM return values. */
@@ -97,13 +105,16 @@ static const struct {
     const char *name;
     int status;
 } RETURNS[] = {
-    { "PAM_AUTH_ERR",         PAM_AUTH_ERR         },
-    { "PAM_AUTHINFO_UNAVAIL", PAM_AUTHINFO_UNAVAIL },
-    { "PAM_IGNORE",           PAM_IGNORE           },
-    { "PAM_NEW_AUTHTOK_REQD", PAM_NEW_AUTHTOK_REQD },
-    { "PAM_SESSION_ERR",      PAM_SESSION_ERR      },
-    { "PAM_SUCCESS",          PAM_SUCCESS          },
-    { "PAM_USER_UNKNOWN",     PAM_USER_UNKNOWN     },
+    /* clang-format off */
+    {"PAM_AUTH_ERR",         PAM_AUTH_ERR        },
+    {"PAM_AUTHINFO_UNAVAIL", PAM_AUTHINFO_UNAVAIL},
+    {"PAM_AUTHTOK_ERR",      PAM_AUTHTOK_ERR     },
+    {"PAM_IGNORE",           PAM_IGNORE          },
+    {"PAM_NEW_AUTHTOK_REQD", PAM_NEW_AUTHTOK_REQD},
+    {"PAM_SESSION_ERR",      PAM_SESSION_ERR     },
+    {"PAM_SUCCESS",          PAM_SUCCESS         },
+    {"PAM_USER_UNKNOWN",     PAM_USER_UNKNOWN    },
+    /* clang-format on */
 };
 
 /* Mapping of PAM prompt styles to their values. */
@@ -111,10 +122,12 @@ static const struct {
     const char *name;
     int style;
 } STYLES[] = {
-    { "echo_off",  PAM_PROMPT_ECHO_OFF },
-    { "echo_on",   PAM_PROMPT_ECHO_ON  },
-    { "error_msg", PAM_ERROR_MSG       },
-    { "info",      PAM_TEXT_INFO       },
+    /* clang-format off */
+    {"echo_off",  PAM_PROMPT_ECHO_OFF},
+    {"echo_on",   PAM_PROMPT_ECHO_ON },
+    {"error_msg", PAM_ERROR_MSG      },
+    {"info",      PAM_TEXT_INFO      },
+    /* clang-format on */
 };
 
 /* Mappings of strings to syslog priorities. */
@@ -122,11 +135,13 @@ static const struct {
     const char *name;
     int priority;
 } PRIORITIES[] = {
-    { "DEBUG",  LOG_DEBUG  },
-    { "INFO",   LOG_INFO   },
-    { "NOTICE", LOG_NOTICE },
-    { "ERR",    LOG_ERR    },
-    { "CRIT",   LOG_CRIT   },
+    /* clang-format off */
+    {"DEBUG",  LOG_DEBUG },
+    {"INFO",   LOG_INFO  },
+    {"NOTICE", LOG_NOTICE},
+    {"ERR",    LOG_ERR   },
+    {"CRIT",   LOG_CRIT  },
+    /* clang-format on */
 };
 
 
@@ -137,7 +152,7 @@ static const struct {
 static char *
 skip_whitespace(char *p)
 {
-    while (isspace((unsigned char)(*p)))
+    while (isspace((unsigned char) (*p)))
         p++;
     return p;
 }
@@ -337,8 +352,16 @@ expand_string(const char *template, const struct script_config *config)
             case 'u':
                 length += strlen(config->user);
                 break;
-            case '0': case '1': case '2': case '3': case '4':
-            case '5': case '6': case '7': case '8': case '9':
+            case '0':
+            case '1':
+            case '2':
+            case '3':
+            case '4':
+            case '5':
+            case '6':
+            case '7':
+            case '8':
+            case '9':
                 if (config->extra[*p - '0'] == NULL)
                     bail("extra script parameter %%%c not set", *p);
                 length += strlen(config->extra[*p - '0']);
@@ -376,8 +399,16 @@ expand_string(const char *template, const struct script_config *config)
                 memcpy(out, config->user, strlen(config->user));
                 out += strlen(config->user);
                 break;
-            case '0': case '1': case '2': case '3': case '4':
-            case '5': case '6': case '7': case '8': case '9':
+            case '0':
+            case '1':
+            case '2':
+            case '3':
+            case '4':
+            case '5':
+            case '6':
+            case '7':
+            case '8':
+            case '9':
                 extra = config->extra[*p - '0'];
                 memcpy(out, extra, strlen(extra));
                 out += strlen(extra);
@@ -560,7 +591,7 @@ static struct output *
 parse_output(FILE *script, const struct script_config *config)
 {
     char *line, *token, *message;
-    struct output *output = NULL;
+    struct output *output;
     int priority;
 
     output = output_new();
